'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = cli;

var _fs = require('fs');

var _stream = require('stream');

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _constants = require('./constants');

var _ = require('.');

var _2 = _interopRequireDefault(_);

var _commands = require('./commands');

var _commands2 = _interopRequireDefault(_commands);

var _file = require('./file');

var _file2 = _interopRequireDefault(_file);

var _package = require('../package.json');

var _settings = require('./settings');

var _settings2 = _interopRequireDefault(_settings);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function cli(errlog) {
    var args = _yargs2.default.env('EZS').usage('Usage: $0 [options] [<file>|<directory>] [<file2> <file3> ...]').version(_package.version).options({
        verbose: {
            alias: 'v',
            default: false,
            describe: 'Enable debug mode with DEBUG=ezs',
            type: 'boolean'
        },
        daemon: {
            alias: 'd',
            describe: 'Launch daemon on a directory containing commands script',
            type: 'string'
        },
        server: {
            alias: 's',
            describe: 'Server to dispach commands',
            type: 'string'
        },
        env: {
            alias: 'e',
            default: false,
            describe: 'Execute commands with environement variables as input',
            type: 'boolean'
        }
    }).epilogue('for more information, find our manual at https://github.com/touv/node-ezs');

    var argv = args.argv;


    if (argv.verbose) {
        _debug2.default.enable('ezs');
    }
    if (argv.daemon) {
        try {
            _settings2.default.servePath = (0, _fs.realpathSync)(argv.daemon);
        } catch (e) {
            errlog('Error: ' + argv.daemon + ' doesn\'t exists.');
            process.exit(1);
        }
        (0, _constants.DEBUG)('Serving ' + _settings2.default.servePath + ' with ' + _settings2.default.nShards + ' shards');
        return _2.default.createCluster();
    }

    if (argv._.length === 0) {
        _yargs2.default.showHelp();
        return process.exit(1);
    }

    var varenvs = void 0;
    var input = void 0;
    if (argv.env) {
        (0, _constants.DEBUG)('Reading environment variables...');
        varenvs = {};
        input = new _stream.PassThrough(_2.default.objectMode());
        input.write(process.env);
        input.end();
    } else {
        (0, _constants.DEBUG)('Reading standard input...');
        varenvs = _extends({}, process.env);
        input = process.stdin;
        input.resume();
        input.setEncoding('utf8');
    }
    var server = Array.isArray(argv.server) ? argv.server : [argv.server];
    var environement = _extends({}, varenvs, { server: server });

    var selectFunc = function selectFunc(func, cmds) {
        if (func === 'pipeline') {
            return _2.default.pipeline(cmds, environement);
        }
        return (0, _2.default)('dispatch', {
            commands: cmds,
            server: server,
            environement: environement
        });
    };
    var runScriptRemote = function runScriptRemote(strm, cmds) {
        (0, _constants.DEBUG)('Connecting to server...');
        var runplan = cmds.analyse();
        var usecmds = cmds.getUseCommands();
        var stream0 = usecmds.reduce(_2.default.command, strm);
        return runplan.reduce(function (stream, section) {
            return stream.pipe(selectFunc(section.func, section.cmds));
        }, stream0);
    };
    var runScriptLocal = function runScriptLocal(strm, cmds) {
        return strm.pipe(_2.default.pipeline(cmds.get(), environement));
    };
    var runScript = function runScript(serverMode) {
        if (serverMode) {
            return runScriptRemote;
        }
        return runScriptLocal;
    };
    var output = argv._.map(function (arg) {
        var script = (0, _file2.default)(_2.default, arg);
        if (!script) {
            errlog('Error: ' + arg + ' isn\'t a file.');
            process.exit(1);
        }
        return script;
    }).map(function (script) {
        return new _commands2.default(_2.default.parseString(script));
    }).reduce(runScript(argv.server), input).on('error', function (e) {
        errlog(e.message.split('\n').shift());
        process.exit(2);
    }).pipe(_2.default.catch(function (e) {
        errlog(e.message.split('\n').shift());
        process.exit(2);
    })).pipe(_2.default.toBuffer());
    output.on('end', function () {
        process.exit(0);
    });
    output.pipe(process.stdout);
    return argv;
}