'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.connectServer = exports.inspectServers = exports.ensureArray = exports.parseAddress = exports.agent = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.writeTo = writeTo;

var _stream = require('stream');

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _parameter = require('./parameter');

var _parameter2 = _interopRequireDefault(_parameter);

var _settings = require('./settings');

var _settings2 = _interopRequireDefault(_settings);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var agent = exports.agent = new _http2.default.Agent({
    maxSockets: 0,
    keepAlive: false,
    timeout: 0
});

var parseAddress = exports.parseAddress = function parseAddress(commands, environment) {
    return function (srvr) {
        if (typeof srvr !== 'string') {
            return null;
        }
        var hostWithPort = srvr.match(/^\[?([^\]]+)\]?:(\d+)$/);
        var serverOptions = {
            hostname: srvr,
            port: _settings2.default.port,
            path: '/',
            method: 'POST',
            headers: {
                'Transfer-Encoding': 'chunked',
                'Content-Type': 'application/json'
            },
            agent: agent
        };
        commands.forEach(function (command, index) {
            serverOptions.headers['X-Command-' + index] = _parameter2.default.encode(JSON.stringify(command));
        });
        Object.keys(environment).forEach(function (keyEnv) {
            serverOptions.headers['X-Environment-' + keyEnv] = _parameter2.default.encode(JSON.stringify(environment[keyEnv]));
        });
        if (hostWithPort) {
            return _extends({}, serverOptions, {
                hostname: hostWithPort[1],
                port: Number(hostWithPort[2])
            });
        }
        return serverOptions;
    };
};

var ensureArray = exports.ensureArray = function ensureArray(a) {
    return Array.isArray(a) ? a : [a];
};

var inspectServers = exports.inspectServers = function inspectServers(servers, commands, environment, ns) {
    return ensureArray(servers).filter(Boolean).filter(function (elem, pos, arr) {
        return arr.indexOf(elem) === pos;
    }).map(parseAddress(commands, environment)).map(function (s) {
        return Array(ns || _settings2.default.nShards).fill(s);
    }) // multiple each line
    .reduce(function (a, b) {
        return a.concat(b);
    }, []);
}; // flatten all

var connectServer = exports.connectServer = function connectServer(ezs) {
    return function (serverOptions, index) {
        var hostname = serverOptions.hostname,
            port = serverOptions.port;

        var connected = false;
        serverOptions.headers = _extends({}, ezs.encodingMode(), serverOptions.headers);
        var input = new _stream.PassThrough(ezs.objectMode());
        var output = new _stream.PassThrough(ezs.objectMode());
        var handle = _http2.default.request(serverOptions, function (res) {
            connected = true;
            (0, _constants.DEBUG)('http://' + hostname + ':' + port + ' send code ' + res.statusCode);
            if (res.statusCode === 200) {
                res.pipe(ezs.uncompress(res.headers)).pipe(ezs('unpack')).pipe(ezs('ungroup')).pipe(output);
                return 1;
            }
            if (res.statusCode === 400) {
                var errmsg = _parameter2.default.decode(res.headers['x-error']);
                output.write(new Error('Server sent: ' + errmsg));
                output.end();
                return 2;
            }
            output.write(new Error('http://' + hostname + ':' + port + ' at item #' + index + ' return ' + res.statusCode));
            return 3;
        });
        handle.on('error', function (e) {
            handle.abort();
            if (!connected) {
                output.write(new Error('http://' + (hostname || '?') + ':' + (port || '?') + ' at item #' + index + ' return ' + e.message));
                return output.end();
            }
            (0, _constants.DEBUG)('http://' + hostname + ':' + port + ' was stopped properly following ' + e);
            return 4;
        });
        handle.setNoDelay(false);

        input.pipe(ezs('group')).pipe(ezs('pack')).pipe(ezs.compress(ezs.encodingMode())).pipe(handle);
        var duplex = [input, output];
        return duplex;
    };
};

function writeTo(stream, data, cb) {
    var check = stream.write(data);
    if (!check) {
        stream.once('drain', cb);
    } else {
        process.nextTick(cb);
    }
    return check;
}