'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.compressStream = compressStream;
exports.uncompressStream = uncompressStream;

var _stream = require('stream');

var _constants = require('./constants');

var resolve = function resolve(name) {
    try {
        require.resolve(name);
        return true;
    } catch (e) {
        return false;
    }
};

var chooseZ = function chooseZ() {
    if (resolve('node-zstd')) {
        // eslint-disable-next-line
        return require('node-zstd');
    }
    if (resolve('zlib')) {
        // eslint-disable-next-line
        return require('zlib');
    }
    return null;
};

var z = chooseZ();

function compressStream(ezs) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var encoding = opts['Content-Encoding'] || opts['content-encoding'] || 'identity';
    if (typeof z.createGunzip === 'function' && encoding === 'gzip') {
        (0, _constants.DEBUG)('ezs will use zlib to compress stream.');
        return z.createGzip();
    }if (typeof z.compressStream === 'function' && encoding === 'zstd') {
        (0, _constants.DEBUG)('ezs will use zstd to compress stream.');
        return z.compressStream();
    }
    (0, _constants.DEBUG)('ezs will not compress stream.');
    return new _stream.PassThrough(ezs.bytesMode());
}

function uncompressStream(ezs) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var encoding = opts['Content-Encoding'] || opts['content-encoding'] || 'identity';
    if (typeof z.createGunzip === 'function' && encoding === 'gzip') {
        (0, _constants.DEBUG)('ezs will use zlib to uncompress stream.');
        return z.createGunzip();
    }if (typeof z.compressStream === 'function' && encoding === 'zstd') {
        (0, _constants.DEBUG)('ezs will use zstd to uncompress stream.');
        return z.decompressStream();
    }
    (0, _constants.DEBUG)('ezs will not uncompress stream.');
    return new _stream.PassThrough(ezs.bytesMode());
}