'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _parameter = require('./parameter');

var _parameter2 = _interopRequireDefault(_parameter);

var _feed = require('./feed');

var _feed2 = _interopRequireDefault(_feed);

var _shell = require('./shell');

var _shell2 = _interopRequireDefault(_shell);

var _constants = require('./constants');

var _SafeTransform2 = require('./SafeTransform');

var _SafeTransform3 = _interopRequireDefault(_SafeTransform2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function createErrorWith(error, index) {
    var stk = error.stack.split('\n');
    var erm = stk.shift();
    var msg = 'Processing item #' + index + ' failed with ' + erm + '\n\t' + stk.slice(0, 10).join('\n\t');
    var err = Error(msg);
    Error.captureStackTrace(err, createErrorWith);
    (0, _constants.DEBUG)('Caught an', err);
    return err;
}

var Engine = function (_SafeTransform) {
    _inherits(Engine, _SafeTransform);

    function Engine(ezs, func, params, environment) {
        _classCallCheck(this, Engine);

        var _this = _possibleConstructorReturn(this, (Engine.__proto__ || Object.getPrototypeOf(Engine)).call(this, ezs.objectMode()));

        _this.func = func;
        _this.funcName = String(func.name);
        _this.index = 0;
        _this.params = params || {};
        _this.scope = {};
        _this.ezs = ezs;
        _this.environment = environment || {};
        _this.nullWasSent = false;
        return _this;
    }

    _createClass(Engine, [{
        key: '_transform',
        value: function _transform(chunk, encoding, done) {
            if (this.nullWasSent) {
                return done();
            }
            this.index += 1;
            if (chunk instanceof Error) {
                this.push(chunk);
                return done();
            }
            return this.execWith(chunk, done);
        }
    }, {
        key: '_flush',
        value: function _flush(done) {
            if (this.nullWasSent) {
                return done();
            }
            this.index += 1;
            return this.execWith(null, done);
        }
    }, {
        key: 'execWith',
        value: function execWith(chunk, done) {
            var _this2 = this;

            var currentIndex = this.index;
            if (chunk === null && currentIndex === 1) {
                this.nullWasSent = true;
                this.push(null);
                return done();
            }
            var warn = function warn(error) {
                _this2.emit('error', createErrorWith(error, currentIndex));
            };
            var push = function push(data) {
                if (data instanceof Error) {
                    (0, _constants.DEBUG)('Ignoring error at item #' + currentIndex);
                    return _this2.push(createErrorWith(data, currentIndex));
                }
                if (data === null) {
                    _this2.nullWasSent = true;
                }
                return _this2.push(data);
            };
            var feed = new _feed2.default(push, done, warn);
            try {
                this.scope.isFirst = function () {
                    return currentIndex === 1;
                };
                this.scope.getIndex = function () {
                    return currentIndex;
                };
                this.scope.isLast = function () {
                    return chunk === null;
                };
                this.scope.getEnv = function (name) {
                    return name === undefined ? _this2.environment : _this2.environment[name];
                };
                this.scope.ezs = this.ezs;
                this.scope.getParam = function (name, defval) {
                    var globalParams = _parameter2.default.get(_this2.ezs, _this2.funcName);
                    if (_this2.params[name] !== undefined) {
                        return (0, _shell2.default)(_this2.params[name], chunk, _this2.environment);
                    }if (globalParams[name] !== undefined) {
                        return globalParams[name];
                    }
                    return defval;
                };
                Promise.resolve(this.func.call(this.scope, chunk, feed, currentIndex)).catch(function (e) {
                    (0, _constants.DEBUG)('Async error thrown at item #' + currentIndex + ', pipeline is broken');
                    _this2.emit('error', createErrorWith(e, currentIndex));
                    done();
                });
            } catch (e) {
                (0, _constants.DEBUG)('Sync error thrown at item #' + currentIndex + ', pipeline carries errors');
                this.push(createErrorWith(e, currentIndex));
                done();
            }
        }
    }]);

    return Engine;
}(_SafeTransform3.default);

exports.default = Engine;