'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _cluster = require('cluster');

var _cluster2 = _interopRequireDefault(_cluster);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _httpShutdown = require('http-shutdown');

var _httpShutdown2 = _interopRequireDefault(_httpShutdown);

var _filenameRegex = require('filename-regex');

var _filenameRegex2 = _interopRequireDefault(_filenameRegex);

var _url = require('url');

var _knownPipeline = require('./knownPipeline');

var _knownPipeline2 = _interopRequireDefault(_knownPipeline);

var _unknownPipeline = require('./unknownPipeline');

var _unknownPipeline2 = _interopRequireDefault(_unknownPipeline);

var _serverInformation = require('./serverInformation');

var _serverInformation2 = _interopRequireDefault(_serverInformation);

var _serverPipeline = require('./serverPipeline');

var _serverPipeline2 = _interopRequireDefault(_serverPipeline);

var _notFound = require('./notFound');

var _notFound2 = _interopRequireDefault(_notFound);

var _parameter = require('../parameter');

var _parameter2 = _interopRequireDefault(_parameter);

var _settings = require('../settings');

var _settings2 = _interopRequireDefault(_settings);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isPipeline = function isPipeline(filename) {
    return filename.match((0, _filenameRegex2.default)()).shift() !== undefined;
};

function createMidddleware(ezs, method, pathname) {
    if (method === 'POST' && pathname === '/') {
        (0, _constants.DEBUG)('Create middleware \'unknownPipeline\' for ' + method + ' ' + pathname);
        return (0, _unknownPipeline2.default)(ezs);
    }
    if (method === 'GET' && pathname === '/') {
        (0, _constants.DEBUG)('Create middleware \'serverInformation\' for ' + method + ' ' + pathname);
        return (0, _serverInformation2.default)(ezs);
    }
    if (method === 'GET' && isPipeline(pathname)) {
        (0, _constants.DEBUG)('Create middleware \'serverPipeline\' for ' + method + ' ' + pathname);
        return (0, _serverPipeline2.default)(ezs);
    }
    if (method === 'POST' && isPipeline(pathname)) {
        (0, _constants.DEBUG)('Create middleware \'knownPipeline\' for ' + method + ' ' + pathname);
        return (0, _knownPipeline2.default)(ezs);
    }
    (0, _constants.DEBUG)('Create middleware \'notFound\' for ' + method + ' ' + pathname);
    return (0, _notFound2.default)();
}

var signals = ['SIGINT', 'SIGTERM'];

function createServer(ezs, port) {
    var serverPort = port || _settings2.default.port;
    var server = (0, _httpShutdown2.default)(_http2.default.createServer(function (request, response) {
        var method = request.method;

        response.socket.setNoDelay(false);
        request.url = (0, _url.parse)(request.url, true);
        var middleware = createMidddleware(ezs, method, request.url.pathname);
        try {
            middleware(request, response);
        } catch (error) {
            (0, _constants.DEBUG)('Server cannot execute commands', error);
            response.writeHead(400, { 'X-error': _parameter2.default.encode(error.toString()) });
            response.end();
        }
        return true;
    }));
    server.setTimeout(0);
    server.listen(serverPort);
    signals.forEach(function (signal) {
        return process.on(signal, function () {
            (0, _constants.DEBUG)('Signal received, stoping server with PID ' + process.pid);
            server.shutdown(function () {
                return process.exit(0);
            });
        });
    });
    (0, _constants.DEBUG)('Server starting with PID ' + process.pid + ' and listening on port ' + serverPort);
    return server;
}

function createCluster(ezs, port) {
    var term = false;
    if (_cluster2.default.isMaster) {
        for (var i = 0; i < _settings2.default.nShards; i += 1) {
            _cluster2.default.fork();
        }
        _cluster2.default.on('exit', function () {
            if (!term) {
                _cluster2.default.fork();
            }
        });
        signals.forEach(function (signal) {
            process.on(signal, function () {
                term = true;
                Object.keys(_cluster2.default.workers).forEach(function (id) {
                    return _cluster2.default.workers[id].kill();
                });
            });
        });
    } else {
        createServer(ezs, port);
    }
    return _cluster2.default;
}

exports.default = {
    createServer: createServer,
    createCluster: createCluster
};