'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _constants = require('../constants');

var _parameter = require('../parameter');

var _parameter2 = _interopRequireDefault(_parameter);

var _file = require('../file');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var knownPipeline = function knownPipeline(ezs) {
    return function (request, response) {
        var _request$url = request.url,
            pathname = _request$url.pathname,
            query = _request$url.query;

        var filePath = ezs.fileToServe(pathname);
        if (!(0, _file.isFile)(filePath)) {
            response.writeHead(404);
            response.end();
            return false;
        }
        var headers = request.headers;

        response.setHeader('Content-Encoding', headers['content-encoding'] || 'identity');
        (0, _constants.DEBUG)('PID ' + process.pid + ' will execute ' + filePath + ' commands with ' + (Object.keys(query).length || 0) + ' global parameters');
        var processor = ezs.fromFile(filePath, query);
        request.pipe(ezs.uncompress(headers)).pipe(processor).pipe(ezs.catch(function (error) {
            (0, _constants.DEBUG)('Server has caught an error', error);
            if (!response.headersSent) {
                response.writeHead(400, { 'X-Error': _parameter2.default.encode(error.toString()) });
                response.end();
            }
        })).pipe(ezs(function (input, output) {
            if (!response.headersSent) {
                response.writeHead(200);
            }
            return output.send(input);
        })).pipe(ezs.toBuffer()).pipe(ezs.compress(headers)).pipe(response);
        request.resume();
        return true;
    };
};

exports.default = knownPipeline;