'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _nodeDir = require('node-dir');

var _nodeDir2 = _interopRequireDefault(_nodeDir);

var _parameter = require('../parameter');

var _parameter2 = _interopRequireDefault(_parameter);

var _settings = require('../settings');

var _settings2 = _interopRequireDefault(_settings);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getInformations = function getInformations(dirPath) {
    return new Promise(function (resolve, reject) {
        _nodeDir2.default.files(dirPath, function (err, files) {
            if (err) {
                return reject(err);
            }
            var scripts = files.filter(function (f) {
                return f.search(/\.(ini|ezs)$/) > 0;
            }).map(function (f) {
                return f.replace(dirPath, '');
            });
            return resolve({
                concurrency: _settings2.default.nShards,
                uptime: Date.now() - _constants.STARTED_AT,
                timestamp: Date.now(),
                version: _constants.VERSION,
                scripts: scripts
            });
        });
    });
};

var serverInformation = function serverInformation(ezs) {
    return function (request, response) {
        getInformations(ezs.fileToServe('')).then(function (informations) {
            var responseBody = JSON.stringify(informations);
            var responseHeaders = {
                'Content-Type': 'application/json',
                'Content-Length': Buffer.byteLength(responseBody)
            };
            response.writeHead(200, responseHeaders);
            response.write(responseBody);
            response.end();
        }).catch(function (error) {
            (0, _constants.DEBUG)('Server has caught an error', error);
            if (!response.headersSent) {
                response.writeHead(500, { 'X-Error': _parameter2.default.encode(error.toString()) });
            }
            response.end();
        });
    };
};

exports.default = serverInformation;