'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _constants = require('../constants');

var _parameter = require('../parameter');

var _parameter2 = _interopRequireDefault(_parameter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var unknownPipeline = function unknownPipeline(ezs) {
    return function (request, response) {
        var headers = request.headers;

        response.setHeader('Content-Encoding', headers['content-encoding'] || 'identity');
        var commands = Object.keys(headers).filter(function (headerKey) {
            return headerKey.indexOf('x-command') === 0;
        }).map(function (headerKey) {
            return parseInt(headerKey.replace('x-command-', ''), 10);
        }).sort().map(function (commandIndex) {
            return _parameter2.default.unscramble(headers['x-command-' + commandIndex]);
        });
        var environment = Object.keys(headers).filter(function (headerKey) {
            return headerKey.indexOf('x-environment') === 0;
        }).map(function (headerKey) {
            return headerKey.replace('x-environment-', '');
        }).map(function (environmentKey) {
            return _defineProperty({}, environmentKey, _parameter2.default.unpack(headers['x-environment-' + environmentKey]));
        }).reduce(function (prev, cur) {
            return Object.assign(prev, cur);
        }, {});
        (0, _constants.DEBUG)('PID ' + process.pid + ' will execute ' + (commands.length || 0) + ' commands with ' + (Object.keys(environment).length || 0) + ' global parameters');
        var processor = ezs.pipeline(commands, environment);
        request.pipe(ezs.uncompress(headers)).pipe(ezs('unpack')).pipe(ezs('ungroup')).pipe(processor).pipe(ezs.catch(function (error) {
            (0, _constants.DEBUG)('Server has caught an error', error);
            if (!response.headersSent) {
                response.writeHead(400, { 'X-Error': _parameter2.default.encode(error.toString()) });
                response.end();
            }
        })).pipe(ezs(function (input, output) {
            if (!response.headersSent) {
                response.writeHead(200);
            }
            return output.send(input);
        })).pipe(ezs('group')).pipe(ezs('pack')).pipe(ezs.compress(headers)).pipe(response);
        request.resume();
    };
};

exports.default = unknownPipeline;