'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _file = require('./file');

var _constants = require('./constants');

var pluginsList = {};

function all() {
    return Object.keys(pluginsList);
}

function get(ezs, plugin, opts) {
    if (plugin === 'use') {
        // very special case : to load statement in the current pipeline
        var args = !Array.isArray(opts) ? [opts] : opts;
        args.forEach(function (arg) {
            if (arg.plugin) {
                var names = arg.plugin.get();
                if (!Array.isArray(names)) {
                    names = [names];
                }
                names.forEach(function (name) {
                    var before1 = Object.keys(pluginsList);
                    var fileName = (0, _file.useFile)(ezs, name);
                    if (!fileName) {
                        throw new Error('\'' + name + '\' is not loaded. It was not found (try to install it).');
                    }
                    // eslint-disable-next-line
                    ezs.use(require(fileName));
                    var after1 = Object.keys(pluginsList);
                    var diff1 = after1.filter(function (item) {
                        return before1.indexOf(item) === -1;
                    });
                    if (diff1.length > 0) {
                        (0, _constants.DEBUG)('These statements are registered: ' + diff1.join(','));
                    }
                });
            }
        });
        return function (data, feed) {
            return feed.send(data);
        };
    }if (typeof plugin === 'function') {
        return plugin;
    }if (typeof plugin === 'string' && pluginsList[plugin]) {
        return pluginsList[plugin];
    }if ((typeof plugin === 'undefined' ? 'undefined' : _typeof(plugin)) === 'object') {
        var firstKey = Object.keys(plugin).slice(0, 1);
        if (typeof plugin[firstKey] === 'function') {
            return plugin[firstKey];
        }
    }
    throw new Error('\'' + plugin + '\' is not loaded. It\'s not a valid statement function.');
}

function set(ezs, plugin) {
    if ((typeof plugin === 'undefined' ? 'undefined' : _typeof(plugin)) !== 'object') {
        throw new Error("Statement is not loaded. It's not a valid plugin (should be an object).");
    }
    var pluginList = plugin.default ? plugin.default : plugin; // ES6 hack
    Object.keys(pluginList).forEach(function (pluginName) {
        if (typeof pluginList[pluginName] === 'function') {
            pluginsList[pluginName] = pluginList[pluginName];
        } else if (_typeof(pluginList[pluginName]) === 'object') {
            ezs.use(pluginList[pluginName]);
        } else {
            throw new Error(pluginName + ' is not loaded. It\'s not a valid plugin.');
        }
    });
    return ezs;
}

function exists(ezs, pluginName) {
    return pluginsList[pluginName] !== undefined;
}

exports.default = {
    get: get,
    set: set,
    all: all,
    exists: exists
};