'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = dispatch;

var _merge = require('merge2');

var _merge2 = _interopRequireDefault(_merge);

var _file = require('../file');

var _file2 = _interopRequireDefault(_file);

var _commands = require('../commands');

var _commands2 = _interopRequireDefault(_commands);

var _constants = require('../constants');

var _client = require('../client');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function dispatch(data, feed) {
    var ezs = this.ezs;

    if (this.isFirst()) {
        this.lastIndex = 0;
        var file = this.getParam('file');
        var script = this.getParam('script', (0, _file2.default)(ezs, file));
        var cmds = new _commands2.default(ezs.parseString(script));
        var commands = this.getParam('commands', cmds.get());
        var servers = (0, _client.inspectServers)(this.getParam('server', []), commands, this.getEnv());

        if (!servers || servers.length === 0 || !commands || commands.length === 0) {
            return feed.stop(new Error('Invalid parmeter for dispatch'));
        }
        var handles = servers.map((0, _client.connectServer)(ezs));
        this.ins = handles.map(function (h) {
            return h[0];
        });
        this.outs = handles.map(function (h) {
            return h[1];
        });
        var funnel = (0, _merge2.default)(this.outs, ezs.objectMode()).on('queueDrain', function () {
            funnel.destroy();
        }).on('error', function (e) {
            return feed.write(e);
        }).on('data', function (d) {
            return feed.write(d);
        });
        this.whenFinish = new Promise(function (resolve) {
            funnel.on('close', resolve);
        });
    }
    if (this.isLast()) {
        this.whenFinish.then(function () {
            return feed.close();
        }).catch(function (e) {
            return feed.stop(e);
        });
        this.ins.forEach(function (handle) {
            return handle.end();
        });
    } else {
        if (this.lastIndex >= this.ins.length) {
            this.lastIndex = 0;
        }
        (0, _constants.DEBUG)('Write #' + this.getIndex() + ' with ' + (data.length || 0) + ' items into handle #' + this.lastIndex + '/' + this.ins.length);
        var check = (0, _client.writeTo)(this.ins[this.lastIndex], data, function () {
            return feed.end();
        });
        if (!check) {
            this.lastIndex += 1;
        }
    }
    return 1;
}